@@ -244,6 +244,7 @@ class MoveScheme(StorableNamedObject):
         self._mover_acceptance = None  # used in analysis
 
     def to_dict(self):
+        self.move_decision_tree()  # always build before save
         ret_dict = {
             'movers': self.movers,
             'network': self.network,
@@ -591,7 +592,6 @@ class MoveScheme(StorableNamedObject):
         check_initial_conditions
         assert_initial_conditions
         """
-
         if sample_set is None:
             sample_set = paths.SampleSet([])
 
@@ -664,15 +664,12 @@ class MoveScheme(StorableNamedObject):
         initial_conditions_report
         """
         (missing, extras) = self.check_initial_conditions(sample_set)
-        msg = ""
-        if len(missing) > 0:
-            msg += "Missing ensembles: " + str(missing) + "\n"
-        if len(extras) > 0 and not allow_extras:
-            msg += "Extra ensembles: " + str(extras) + "\n"
-        if msg != "":
+        if len(missing) > 0 or (len(extras) > 0 and not allow_extras):
+            msg = self.initial_conditions_report(sample_set,
+                                                 report_correct=False)
             raise AssertionError("Bad initial conditions.\n" + msg)
 
-    def initial_conditions_report(self, sample_set):
+    def initial_conditions_report(self, sample_set, report_correct=True):
         """
         String report on whether the given SampleSet gives good initial
         conditions.
@@ -684,6 +681,9 @@ class MoveScheme(StorableNamedObject):
         ----------
         sample_set : :class:`.SampleSet`
             proposed set of initial conditions for this movescheme
+        report_correct : bool
+            whether to report when there are no missing/extra ensembles;
+            default True
 
         Returns
         -------
@@ -693,16 +693,16 @@ class MoveScheme(StorableNamedObject):
         """
         (missing, extra) = self.check_initial_conditions(sample_set)
         msg = ""
-        if len(missing) == 0:
+        if len(missing) == 0 and report_correct:
             msg += "No missing ensembles.\n"
-        else:
+        elif len(missing) > 0:
             msg += "Missing ensembles:\n"
             for ens_list in missing:
-                msg += "*  [" 
+                msg += "*  ["
                 msg += ", ".join([ens.name for ens in ens_list]) + "]\n"
-        if len(extra) == 0:
+        if len(extra) == 0 and report_correct:
             msg += "No extra ensembles.\n"
-        else:
+        elif len(extra) > 0:
             msg += "Extra ensembles:\n"
             for ens in extra:
                 msg += "*  " + ens.name + "\n"
